@@ -1,5 +1,6 @@
 /* GStreamer
  * Copyright (C) 2005 Wim Taymans <wim@fluendo.com>
+ * Copyright (C) 2006 Tim-Philipp Müller <tim centricular net>
  *
  * gstalsasink.c:
  *
@@ -33,7 +34,7 @@
  * Play an Ogg/Vorbis file.
  * </para>
  * <programlisting>
- * gst-launch -v filesrc location=sine.ogg ! oggdemux ! vorbisdec ! audioconvert ! alsasink
+ * gst-launch -v filesrc location=sine.ogg ! oggdemux ! vorbisdec ! audioconvert ! audioresample ! alsasink
  * </programlisting>
  * </refsect2>
  *
@@ -53,9 +54,9 @@
 
 #include "gstalsa.h"
 #include "gstalsasink.h"
+#include "gstalsadeviceprobe.h"
 
 #include <gst/gst-i18n-plugin.h>
-#include <gst/audio/multichannel.h>
 
 /* elementfactory information */
 static const GstElementDetails gst_alsasink_details =
@@ -74,10 +75,11 @@ enum
   PROP_DEVICE_NAME
 };
 
-static void gst_alsasink_base_init (gpointer g_class);
-static void gst_alsasink_class_init (GstAlsaSinkClass * klass);
-static void gst_alsasink_init (GstAlsaSink * alsasink);
-static void gst_alsasink_dispose (GObject * object);
+static void gst_alsasink_init_interfaces (GType type);
+
+GST_BOILERPLATE_FULL (GstAlsaSink, gst_alsasink, GstAudioSink,
+    GST_TYPE_AUDIO_SINK, gst_alsasink_init_interfaces);
+
 static void gst_alsasink_finalise (GObject * object);
 static void gst_alsasink_set_property (GObject * object,
     guint prop_id, const GValue * value, GParamSpec * pspec);
@@ -96,11 +98,10 @@ static guint gst_alsasink_write (GstAudioSink * asink, gpointer data,
 static guint gst_alsasink_delay (GstAudioSink * asink);
 static void gst_alsasink_reset (GstAudioSink * asink);
 
-/* AlsaSink signals and args */
-enum
-{
-  LAST_SIGNAL
-};
+static gint output_ref;         /* 0    */
+static snd_output_t *output;    /* NULL */
+static GStaticMutex output_mutex = G_STATIC_MUTEX_INIT;
+
 
 #if (G_BYTE_ORDER == G_LITTLE_ENDIAN)
 # define ALSA_SINK_FACTORY_ENDIANNESS	"LITTLE_ENDIAN, BIG_ENDIAN"
@@ -117,54 +118,20 @@ static GstStaticPadTemplate alsasink_sink_factory =
         "signed = (boolean) { TRUE, FALSE }, "
         "width = (int) 32, "
         "depth = (int) 32, "
-        "rate = (int) [ 1, MAX ], " "channels = (int) [ 1, 8 ]; "
+        "rate = (int) [ 1, MAX ], " "channels = (int) [ 1, MAX ]; "
         "audio/x-raw-int, "
         "endianness = (int) { " ALSA_SINK_FACTORY_ENDIANNESS " }, "
         "signed = (boolean) { TRUE, FALSE }, "
         "width = (int) 16, "
         "depth = (int) 16, "
-        "rate = (int) [ 1, MAX ], " "channels = (int) [ 1, 8 ]; "
+        "rate = (int) [ 1, MAX ], " "channels = (int) [ 1, MAX ]; "
         "audio/x-raw-int, "
         "signed = (boolean) { TRUE, FALSE }, "
         "width = (int) 8, "
         "depth = (int) 8, "
-        "rate = (int) [ 1, MAX ], " "channels = (int) [ 1, 8 ]")
+        "rate = (int) [ 1, MAX ], " "channels = (int) [ 1, MAX ]")
     );
 
-static GstElementClass *parent_class = NULL;
-
-GType
-gst_alsasink_get_type (void)
-{
-  static GType alsasink_type = 0;
-
-  if (!alsasink_type) {
-    static const GTypeInfo alsasink_info = {
-      sizeof (GstAlsaSinkClass),
-      gst_alsasink_base_init,
-      NULL,
-      (GClassInitFunc) gst_alsasink_class_init,
-      NULL,
-      NULL,
-      sizeof (GstAlsaSink),
-      0,
-      (GInstanceInitFunc) gst_alsasink_init,
-    };
-
-    alsasink_type =
-        g_type_register_static (GST_TYPE_AUDIO_SINK, "GstAlsaSink",
-        &alsasink_info, 0);
-  }
-
-  return alsasink_type;
-}
-
-static void
-gst_alsasink_dispose (GObject * object)
-{
-  G_OBJECT_CLASS (parent_class)->dispose (object);
-}
-
 static void
 gst_alsasink_finalise (GObject * object)
 {
@@ -173,9 +140,23 @@ gst_alsasink_finalise (GObject * object)
   g_free (sink->device);
   g_mutex_free (sink->alsa_lock);
 
+  g_static_mutex_lock (&output_mutex);
+  --output_ref;
+  if (output_ref == 0) {
+    snd_output_close (output);
+    output = NULL;
+  }
+  g_static_mutex_unlock (&output_mutex);
+
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
+static void
+gst_alsasink_init_interfaces (GType type)
+{
+  gst_alsa_type_add_device_property_probe_interface (type);
+}
+
 static void
 gst_alsasink_base_init (gpointer g_class)
 {
@@ -203,7 +184,6 @@ gst_alsasink_class_init (GstAlsaSinkClass * klass)
 
   parent_class = g_type_class_peek_parent (klass);
 
-  gobject_class->dispose = GST_DEBUG_FUNCPTR (gst_alsasink_dispose);
   gobject_class->finalize = GST_DEBUG_FUNCPTR (gst_alsasink_finalise);
   gobject_class->get_property = GST_DEBUG_FUNCPTR (gst_alsasink_get_property);
   gobject_class->set_property = GST_DEBUG_FUNCPTR (gst_alsasink_set_property);
@@ -239,9 +219,12 @@ gst_alsasink_set_property (GObject * object, guint prop_id,
 
   switch (prop_id) {
     case PROP_DEVICE:
-      if (sink->device)
-        g_free (sink->device);
-      sink->device = g_strdup (g_value_get_string (value));
+      g_free (sink->device);
+      sink->device = g_value_dup_string (value);
+      /* setting NULL restores the default device */
+      if (sink->device == NULL) {
+        sink->device = g_strdup (DEFAULT_DEVICE);
+      }
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -262,16 +245,9 @@ gst_alsasink_get_property (GObject * object, guint prop_id,
       g_value_set_string (value, sink->device);
       break;
     case PROP_DEVICE_NAME:
-      if (sink->handle) {
-        snd_pcm_info_t *info;
-
-        snd_pcm_info_malloc (&info);
-        snd_pcm_info (sink->handle, info);
-        g_value_set_string (value, snd_pcm_info_get_name (info));
-        snd_pcm_info_free (info);
-      } else {
-        g_value_set_string (value, NULL);
-      }
+      g_value_take_string (value,
+          gst_alsa_find_device_name (GST_OBJECT_CAST (sink),
+              sink->device, sink->handle, SND_PCM_STREAM_PLAYBACK));
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -279,10 +255,8 @@ gst_alsasink_get_property (GObject * object, guint prop_id,
   }
 }
 
-static snd_output_t *output = NULL;
-
 static void
-gst_alsasink_init (GstAlsaSink * alsasink)
+gst_alsasink_init (GstAlsaSink * alsasink, GstAlsaSinkClass * g_class)
 {
   GST_DEBUG_OBJECT (alsasink, "initializing alsasink");
 
@@ -291,7 +265,12 @@ gst_alsasink_init (GstAlsaSink * alsasink)
   alsasink->cached_caps = NULL;
   alsasink->alsa_lock = g_mutex_new ();
 
-  snd_output_stdio_attach (&output, stdout, 0);
+  g_static_mutex_lock (&output_mutex);
+  if (output_ref == 0) {
+    snd_output_stdio_attach (&output, stdout, 0);
+    ++output_ref;
+  }
+  g_static_mutex_unlock (&output_mutex);
 }
 
 #define CHECK(call, error) \
@@ -300,106 +279,13 @@ if ((err = call) < 0)           \
   goto error;                   \
 } G_STMT_END;
 
-/* we don't have channel mappings for more than this many channels */
-#define GST_ALSA_MAX_CHANNELS 8
-
-static GstStructure *
-get_channel_free_structure (const GstStructure * in_structure)
-{
-  GstStructure *s = gst_structure_copy (in_structure);
-
-  gst_structure_remove_field (s, "channels");
-  return s;
-}
-
-static void
-caps_add_channel_configuration (GstCaps * caps,
-    const GstStructure * in_structure, gint min_channels, gint max_channels)
-{
-  GstAudioChannelPosition pos[8] = {
-    GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT,
-    GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT,
-    GST_AUDIO_CHANNEL_POSITION_REAR_LEFT,
-    GST_AUDIO_CHANNEL_POSITION_REAR_RIGHT,
-    GST_AUDIO_CHANNEL_POSITION_FRONT_CENTER,
-    GST_AUDIO_CHANNEL_POSITION_LFE,
-    GST_AUDIO_CHANNEL_POSITION_SIDE_LEFT,
-    GST_AUDIO_CHANNEL_POSITION_SIDE_RIGHT
-  };
-  GstStructure *s = NULL;
-  gint c;
-
-  if (min_channels == max_channels) {
-    s = get_channel_free_structure (in_structure);
-    gst_structure_set (s, "channels", G_TYPE_INT, max_channels, NULL);
-    gst_caps_append_structure (caps, s);
-    return;
-  }
-
-  g_assert (min_channels >= 1);
-
-  /* mono and stereo don't need channel configurations */
-  if (min_channels == 2) {
-    s = get_channel_free_structure (in_structure);
-    gst_structure_set (s, "channels", G_TYPE_INT, 2, NULL);
-    gst_caps_append_structure (caps, s);
-  } else if (min_channels == 1 && max_channels >= 2) {
-    s = get_channel_free_structure (in_structure);
-    gst_structure_set (s, "channels", GST_TYPE_INT_RANGE, 1, 2, NULL);
-    gst_caps_append_structure (caps, s);
-  }
-
-  /* don't know whether to use 2.1 or 3.0 here - but I suspect
-   * alsa might work around that/fix it somehow. Can we tell alsa
-   * what our channel layout is like? */
-  if (max_channels >= 3) {
-    GstAudioChannelPosition pos_21[3] = {
-      GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT,
-      GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT,
-      GST_AUDIO_CHANNEL_POSITION_LFE
-    };
-
-    s = get_channel_free_structure (in_structure);
-    gst_structure_set (s, "channels", G_TYPE_INT, 3, NULL);
-    gst_audio_set_channel_positions (s, pos_21);
-    gst_caps_append_structure (caps, s);
-  }
-
-  /* everything else (4, 6, 8 channels) needs a channel layout */
-  for (c = 4; c < 8; c += 2) {
-    if (max_channels >= c) {
-      s = get_channel_free_structure (in_structure);
-      gst_structure_set (s, "channels", G_TYPE_INT, c, NULL);
-      gst_audio_set_channel_positions (s, pos);
-      gst_caps_append_structure (caps, s);
-    }
-  }
-}
-
 static GstCaps *
 gst_alsasink_getcaps (GstBaseSink * bsink)
 {
-  snd_pcm_format_mask_t *mask;
-  snd_pcm_hw_params_t *hw_params;
   GstElementClass *element_class;
   GstPadTemplate *pad_template;
   GstAlsaSink *sink = GST_ALSA_SINK (bsink);
-  GstCaps *tmpl_caps;
-  GstCaps *caps = NULL;
-  GstStructure *s;
-  guint min, max;
-  gint i, err, width;
-  gint min_channels, max_channels;
-  guint bits = 0;
-  static const int audio_fmts[] = {
-    SND_PCM_FORMAT_U8, SND_PCM_FORMAT_S8,
-    SND_PCM_FORMAT_S16, SND_PCM_FORMAT_U16,
-    /*SND_PCM_FORMAT_S24, SND_PCM_FORMAT_U24, */
-    SND_PCM_FORMAT_S32, SND_PCM_FORMAT_U32
-  };
-  static const guint audio_bits[] = {
-    8, 8, 16, 16, 32, 32
-  };
+  GstCaps *caps;
 
   if (sink->handle == NULL) {
     GST_DEBUG_OBJECT (sink, "device not open, using template caps");
@@ -407,99 +293,24 @@ gst_alsasink_getcaps (GstBaseSink * bsink)
   }
 
   if (sink->cached_caps) {
-    GST_DEBUG_OBJECT (sink, "Returning cached caps %" GST_PTR_FORMAT,
-        sink->cached_caps);
+    GST_LOG_OBJECT (sink, "Returning cached caps");
     return gst_caps_ref (sink->cached_caps);
   }
 
-  snd_pcm_hw_params_alloca (&hw_params);
-  CHECK (snd_pcm_hw_params_any (sink->handle, hw_params), error);
-
-  GST_LOG_OBJECT (sink, "probing channels ...");
-  CHECK (snd_pcm_hw_params_get_channels_min (hw_params, &min), min_chan_error);
-  CHECK (snd_pcm_hw_params_get_channels_max (hw_params, &max), max_chan_error);
-
-  min_channels = min;
-  max_channels = max;
-
-  if (min_channels < 0) {       /* hmm? min and max are unsigned */
-    min_channels = 1;
-    max_channels = GST_ALSA_MAX_CHANNELS;
-  } else if (max_channels < 0) {        /* hmm? min and max are unsigned */
-    max_channels = GST_ALSA_MAX_CHANNELS;
-  }
-
-  if (min_channels > max_channels) {
-    gint temp;
-
-    GST_WARNING_OBJECT (sink, "minimum channels > maximum channels (%d > %d), "
-        "please fix your soundcard drivers", min, max);
-    temp = min_channels;
-    min_channels = max_channels;
-    max_channels = temp;
-  }
-
-  min_channels = MAX (min_channels, 1);
-  max_channels = MIN (GST_ALSA_MAX_CHANNELS, max_channels);
-
-  GST_LOG_OBJECT (sink, "Min. channels = %d (%d)", min_channels, min);
-  GST_LOG_OBJECT (sink, "Max. channels = %d (%d)", max_channels, max);
-
-  snd_pcm_format_mask_alloca (&mask);
-  snd_pcm_hw_params_get_format_mask (hw_params, mask);
-  for (i = 0; i < 6; i++) {
-    if (snd_pcm_format_mask_test (mask, audio_fmts[i])) {
-      bits |= audio_bits[i];
-    }
-  }
-  GST_LOG_OBJECT (sink, "Bits = 0x%08x", bits);
-
-  /* fill caps according to capabilities gathered above */
-
   element_class = GST_ELEMENT_GET_CLASS (sink);
   pad_template = gst_element_class_get_pad_template (element_class, "sink");
-
   g_return_val_if_fail (pad_template != NULL, NULL);
 
-  tmpl_caps = gst_pad_template_get_caps (pad_template);
+  caps = gst_alsa_probe_supported_formats (GST_OBJECT (sink), sink->handle,
+      gst_pad_template_get_caps (pad_template));
 
-  caps = gst_caps_new_empty ();
-
-  for (i = 0; i < gst_caps_get_size (tmpl_caps); ++i) {
-    s = gst_caps_get_structure (tmpl_caps, i);
-    gst_structure_get_int (s, "width", &width);
-    /* TODO: filter signed/unsigned */
-    if (bits & width) {
-      caps_add_channel_configuration (caps, s, min_channels, max_channels);
-    } else {
-      GST_LOG_OBJECT (sink, "width = %d unsupported", width);
-    }
+  if (caps) {
+    sink->cached_caps = gst_caps_ref (caps);
   }
 
-  sink->cached_caps = gst_caps_ref (caps);
-
-  GST_DEBUG_OBJECT (sink, "returning caps %" GST_PTR_FORMAT, caps);
+  GST_INFO_OBJECT (sink, "returning caps %" GST_PTR_FORMAT, caps);
 
   return caps;
-
-error:
-  {
-    GST_ERROR_OBJECT (sink, "failed to query alsasink formats: %s",
-        snd_strerror (err));
-    return NULL;
-  }
-min_chan_error:
-  {
-    GST_ERROR_OBJECT (sink, "failed to query minimum channel count: %s",
-        snd_strerror (err));
-    return NULL;
-  }
-max_chan_error:
-  {
-    GST_ERROR_OBJECT (sink, "failed to query maximum channel count: %s",
-        snd_strerror (err));
-    return NULL;
-  }
 }
 
 static int
@@ -508,12 +319,19 @@ set_hwparams (GstAlsaSink * alsa)
   guint rrate;
   gint err, dir;
   snd_pcm_hw_params_t *params;
+  guint period_time, buffer_time;
 
   snd_pcm_hw_params_alloca (&params);
 
-  GST_DEBUG_OBJECT (alsa, "Negotiating to %d channels @ %d Hz (format = %d)",
-      alsa->channels, alsa->rate, alsa->format);
+  GST_DEBUG_OBJECT (alsa, "Negotiating to %d channels @ %d Hz (format = %s)",
+      alsa->channels, alsa->rate, snd_pcm_format_name (alsa->format));
 
+  /* start with requested values, if we cannot configure alsa for those values,
+   * we set these values to -1, which will leave the default alsa values */
+  buffer_time = alsa->buffer_time;
+  period_time = alsa->period_time;
+
+retry:
   /* choose all parameters */
   CHECK (snd_pcm_hw_params_any (alsa->handle, params), no_config);
   /* set the interleaved read/write format */
@@ -532,26 +350,67 @@ set_hwparams (GstAlsaSink * alsa)
   if (rrate != alsa->rate)
     goto rate_match;
 
-  if (alsa->buffer_time != -1) {
-    /* set the buffer time */
-    CHECK (snd_pcm_hw_params_set_buffer_time_near (alsa->handle, params,
-            &alsa->buffer_time, &dir), buffer_time);
+  /* get and dump some limits */
+  {
+    guint min, max;
+
+    snd_pcm_hw_params_get_buffer_time_min (params, &min, &dir);
+    snd_pcm_hw_params_get_buffer_time_max (params, &max, &dir);
+
+    GST_DEBUG_OBJECT (alsa, "buffer time %u, min %u, max %u",
+        alsa->buffer_time, min, max);
+
+    snd_pcm_hw_params_get_period_time_min (params, &min, &dir);
+    snd_pcm_hw_params_get_period_time_max (params, &max, &dir);
+
+    GST_DEBUG_OBJECT (alsa, "period time %u, min %u, max %u",
+        alsa->period_time, min, max);
+
+    snd_pcm_hw_params_get_periods_min (params, &min, &dir);
+    snd_pcm_hw_params_get_periods_max (params, &max, &dir);
+
+    GST_DEBUG_OBJECT (alsa, "periods min %u, max %u", min, max);
   }
-  if (alsa->period_time != -1) {
+
+  /* now try to configure the buffer time and period time, if one
+   * of those fail, we fall back to the defaults and emit a warning. */
+  if (buffer_time != -1) {
+    /* set the buffer time */
+    if ((err = snd_pcm_hw_params_set_buffer_time_near (alsa->handle, params,
+                &buffer_time, &dir)) < 0) {
+      GST_ELEMENT_WARNING (alsa, RESOURCE, SETTINGS, (NULL),
+          ("Unable to set buffer time %i for playback: %s",
+              buffer_time, snd_strerror (err)));
+      /* disable buffer_time the next round */
+      buffer_time = -1;
+      goto retry;
+    }
+  }
+  if (period_time != -1) {
     /* set the period time */
-    CHECK (snd_pcm_hw_params_set_period_time_near (alsa->handle, params,
-            &alsa->period_time, &dir), period_time);
+    if ((err = snd_pcm_hw_params_set_period_time_near (alsa->handle, params,
+                &period_time, &dir)) < 0) {
+      GST_ELEMENT_WARNING (alsa, RESOURCE, SETTINGS, (NULL),
+          ("Unable to set period time %i for playback: %s",
+              period_time, snd_strerror (err)));
+      /* disable period_time the next round */
+      period_time = -1;
+      goto retry;
+    }
   }
 
   /* write the parameters to device */
   CHECK (snd_pcm_hw_params (alsa->handle, params), set_hw_params);
 
+  /* now get the configured values */
   CHECK (snd_pcm_hw_params_get_buffer_size (params, &alsa->buffer_size),
       buffer_size);
-
   CHECK (snd_pcm_hw_params_get_period_size (params, &alsa->period_size, &dir),
       period_size);
 
+  GST_DEBUG_OBJECT (alsa, "buffer size %lu, period size %lu", alsa->buffer_size,
+      alsa->period_size);
+
   return 0;
 
   /* ERRORS */
@@ -604,26 +463,12 @@ rate_match:
         ("Rate doesn't match (requested %iHz, get %iHz)", alsa->rate, err));
     return -EINVAL;
   }
-buffer_time:
-  {
-    GST_ELEMENT_ERROR (alsa, RESOURCE, SETTINGS, (NULL),
-        ("Unable to set buffer time %i for playback: %s",
-            alsa->buffer_time, snd_strerror (err)));
-    return err;
-  }
 buffer_size:
   {
     GST_ELEMENT_ERROR (alsa, RESOURCE, SETTINGS, (NULL),
         ("Unable to get buffer size for playback: %s", snd_strerror (err)));
     return err;
   }
-period_time:
-  {
-    GST_ELEMENT_ERROR (alsa, RESOURCE, SETTINGS, (NULL),
-        ("Unable to set period time %i for playback: %s", alsa->period_time,
-            snd_strerror (err)));
-    return err;
-  }
 period_size:
   {
     GST_ELEMENT_ERROR (alsa, RESOURCE, SETTINGS, (NULL),
@@ -765,6 +610,7 @@ gst_alsasink_open (GstAudioSink * asink)
 
   CHECK (snd_pcm_open (&alsa->handle, alsa->device, SND_PCM_STREAM_PLAYBACK,
           SND_PCM_NONBLOCK), open_error);
+  GST_LOG_OBJECT (alsa, "Opened device %s", alsa->device);
 
   return TRUE;
 
@@ -772,10 +618,12 @@ gst_alsasink_open (GstAudioSink * asink)
 open_error:
   {
     if (err == -EBUSY) {
-      GST_ELEMENT_ERROR (alsa, RESOURCE, BUSY, (NULL), ("Device is busy"));
+      GST_ELEMENT_ERROR (alsa, RESOURCE, BUSY, (NULL), ("Device '%s' is busy",
+              alsa->device));
     } else {
       GST_ELEMENT_ERROR (alsa, RESOURCE, OPEN_WRITE,
-          (NULL), ("Playback open error: %s", snd_strerror (err)));
+          (NULL), ("Playback open error on device '%s': %s", alsa->device,
+              snd_strerror (err)));
     }
     return FALSE;
   }
@@ -800,10 +648,6 @@ gst_alsasink_prepare (GstAudioSink * asink, GstRingBufferSpec * spec)
   alsa->bytes_per_sample = spec->bytes_per_sample;
   spec->segsize = alsa->period_size * spec->bytes_per_sample;
   spec->segtotal = alsa->buffer_size / alsa->period_size;
-  spec->silence_sample[0] = 0;
-  spec->silence_sample[1] = 0;
-  spec->silence_sample[2] = 0;
-  spec->silence_sample[3] = 0;
 
   return TRUE;
 
@@ -897,14 +741,15 @@ close_error:
  *   Underrun and suspend recovery
  */
 static gint
-xrun_recovery (snd_pcm_t * handle, gint err)
+xrun_recovery (GstAlsaSink * alsa, snd_pcm_t * handle, gint err)
 {
-  GST_DEBUG ("xrun recovery %d", err);
+  GST_DEBUG_OBJECT (alsa, "xrun recovery %d", err);
 
   if (err == -EPIPE) {          /* under-run */
     err = snd_pcm_prepare (handle);
     if (err < 0)
-      GST_WARNING ("Can't recovery from underrun, prepare failed: %s",
+      GST_WARNING_OBJECT (alsa,
+          "Can't recovery from underrun, prepare failed: %s",
           snd_strerror (err));
     return 0;
   } else if (err == -ESTRPIPE) {
@@ -914,7 +759,8 @@ xrun_recovery (snd_pcm_t * handle, gint err)
     if (err < 0) {
       err = snd_pcm_prepare (handle);
       if (err < 0)
-        GST_WARNING ("Can't recovery from suspend, prepare failed: %s",
+        GST_WARNING_OBJECT (alsa,
+            "Can't recovery from suspend, prepare failed: %s",
             snd_strerror (err));
     }
     return 0;
@@ -935,7 +781,7 @@ gst_alsasink_write (GstAudioSink * asink, gpointer data, guint length)
   cptr = length / alsa->bytes_per_sample;
   ptr = data;
 
-  GST_ALSA_LOCK (asink);
+  GST_ALSA_SINK_LOCK (asink);
   while (cptr > 0) {
     err = snd_pcm_writei (alsa->handle, ptr, cptr);
 
@@ -944,7 +790,7 @@ gst_alsasink_write (GstAudioSink * asink, gpointer data, guint length)
       GST_DEBUG_OBJECT (asink, "Write error: %s", snd_strerror (err));
       if (err == -EAGAIN) {
         continue;
-      } else if (xrun_recovery (alsa->handle, err) < 0) {
+      } else if (xrun_recovery (alsa, alsa->handle, err) < 0) {
         goto write_error;
       }
       continue;
@@ -953,13 +799,13 @@ gst_alsasink_write (GstAudioSink * asink, gpointer data, guint length)
     ptr += err * alsa->channels;
     cptr -= err;
   }
-  GST_ALSA_UNLOCK (asink);
+  GST_ALSA_SINK_UNLOCK (asink);
 
   return length - cptr;
 
 write_error:
   {
-    GST_ALSA_UNLOCK (asink);
+    GST_ALSA_SINK_UNLOCK (asink);
     return length;              /* skip one period */
   }
 }
@@ -985,29 +831,29 @@ gst_alsasink_reset (GstAudioSink * asink)
 
   alsa = GST_ALSA_SINK (asink);
 
-  GST_ALSA_LOCK (asink);
+  GST_ALSA_SINK_LOCK (asink);
   GST_DEBUG_OBJECT (alsa, "drop");
   CHECK (snd_pcm_drop (alsa->handle), drop_error);
   GST_DEBUG_OBJECT (alsa, "prepare");
   CHECK (snd_pcm_prepare (alsa->handle), prepare_error);
   GST_DEBUG_OBJECT (alsa, "reset done");
-  GST_ALSA_UNLOCK (asink);
+  GST_ALSA_SINK_UNLOCK (asink);
 
   return;
 
   /* ERRORS */
 drop_error:
   {
-    GST_ELEMENT_ERROR (alsa, RESOURCE, SETTINGS,
-        ("alsa-reset: pcm drop error: %s", snd_strerror (err)), (NULL));
-    GST_ALSA_UNLOCK (asink);
+    GST_ERROR_OBJECT (alsa, "alsa-reset: pcm drop error: %s",
+        snd_strerror (err));
+    GST_ALSA_SINK_UNLOCK (asink);
     return;
   }
 prepare_error:
   {
-    GST_ELEMENT_ERROR (alsa, RESOURCE, SETTINGS,
-        ("alsa-reset: pcm prepare error: %s", snd_strerror (err)), (NULL));
-    GST_ALSA_UNLOCK (asink);
+    GST_ERROR_OBJECT (alsa, "alsa-reset: pcm prepare error: %s",
+        snd_strerror (err));
+    GST_ALSA_SINK_UNLOCK (asink);
     return;
   }
 }
