@@ -61,18 +61,15 @@
 
 #include <stdio.h>
 #include <errno.h>
+#include <unistd.h>
 #include <stdlib.h>
 #include <string.h>
+#include <fcntl.h>
 #include <time.h>
 
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-
 
 /* we include this here to get the G_OS_* defines */
 #include <glib.h>
-#include <gst/gst.h>
 
 #ifdef G_OS_WIN32
 #include <winsock2.h>
@@ -83,7 +80,6 @@
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
-#include <fcntl.h>
 #endif
 
 #ifdef HAVE_FIONREAD_IN_SYS_FILIO
@@ -93,26 +89,31 @@
 #include "gstrtspconnection.h"
 #include "gstrtspbase64.h"
 
+/* the select call is also performed on the control sockets, that way
+ * we can send special commands to unlock or restart the select call */
+#define CONTROL_RESTART        'R'      /* restart the select call */
+#define CONTROL_STOP           'S'      /* stop the select call */
+#define CONTROL_SOCKETS(conn)  conn->control_sock
+#define WRITE_SOCKET(conn)     conn->control_sock[1]
+#define READ_SOCKET(conn)      conn->control_sock[0]
+
+#define SEND_COMMAND(conn, command)              \
+G_STMT_START {                                  \
+  unsigned char c; c = command;                 \
+  write (WRITE_SOCKET(conn), &c, 1);             \
+} G_STMT_END
+
+#define READ_COMMAND(conn, command, res)         \
+G_STMT_START {                                  \
+  res = read(READ_SOCKET(conn), &command, 1);    \
+} G_STMT_END
+
 #ifdef G_OS_WIN32
-#define FIONREAD_TYPE gulong
 #define IOCTL_SOCKET ioctlsocket
-#define READ_SOCKET(fd, buf, len) recv (fd, buf, len, 0)
-#define WRITE_SOCKET(fd, buf, len) send (fd, buf, len, 0)
-#define CLOSE_SOCKET(sock) closesocket (sock)
-#define ERRNO_IS_NOT_EAGAIN (WSAGetLastError () != WSAEWOULDBLOCK)
-#define ERRNO_IS_NOT_EINTR (WSAGetLastError () != WSAEINTR)
-/* According to Microsoft's connect() documentation this one returns
- * WSAEWOULDBLOCK and not WSAEINPROGRESS. */
-#define ERRNO_IS_NOT_EINPROGRESS (WSAGetLastError () != WSAEWOULDBLOCK)
+#define CLOSE_SOCKET(sock) closesocket(sock);
 #else
-#define FIONREAD_TYPE gint
 #define IOCTL_SOCKET ioctl
-#define READ_SOCKET(fd, buf, len) read (fd, buf, len)
-#define WRITE_SOCKET(fd, buf, len) write (fd, buf, len)
-#define CLOSE_SOCKET(sock) close (sock)
-#define ERRNO_IS_NOT_EAGAIN (errno != EAGAIN)
-#define ERRNO_IS_NOT_EINTR (errno != EINTR)
-#define ERRNO_IS_NOT_EINPROGRESS (errno != EINPROGRESS)
+#define CLOSE_SOCKET(sock) close(sock);
 #endif
 
 #ifdef G_OS_WIN32
@@ -145,17 +146,36 @@ inet_aton (const char *c, struct in_addr *paddr)
 GstRTSPResult
 gst_rtsp_connection_create (GstRTSPUrl * url, GstRTSPConnection ** conn)
 {
+  gint ret;
   GstRTSPConnection *newconn;
 
+#ifdef G_OS_WIN32
+  unsigned long flags;
+#endif /* G_OS_WIN32 */
+
   g_return_val_if_fail (conn != NULL, GST_RTSP_EINVAL);
 
   newconn = g_new0 (GstRTSPConnection, 1);
 
-  if ((newconn->fdset = gst_poll_new (TRUE)) == NULL)
-    goto no_fdset;
+#ifdef G_OS_WIN32
+  /* This should work on UNIX too. PF_UNIX sockets replaced with pipe */
+  /* pipe( CONTROL_SOCKETS(newconn) ) */
+  if ((ret = pipe (CONTROL_SOCKETS (newconn))) < 0)
+    goto no_socket_pair;
+
+  ioctlsocket (READ_SOCKET (newconn), FIONBIO, &flags);
+  ioctlsocket (WRITE_SOCKET (newconn), FIONBIO, &flags);
+#else
+  if ((ret =
+          socketpair (PF_UNIX, SOCK_STREAM, 0, CONTROL_SOCKETS (newconn))) < 0)
+    goto no_socket_pair;
+
+  fcntl (READ_SOCKET (newconn), F_SETFL, O_NONBLOCK);
+  fcntl (WRITE_SOCKET (newconn), F_SETFL, O_NONBLOCK);
+#endif
 
   newconn->url = url;
-  newconn->fd.fd = -1;
+  newconn->fd = -1;
   newconn->timer = g_timer_new ();
 
   newconn->auth_method = GST_RTSP_AUTH_NONE;
@@ -167,7 +187,7 @@ gst_rtsp_connection_create (GstRTSPUrl * url, GstRTSPConnection ** conn)
   return GST_RTSP_OK;
 
   /* ERRORS */
-no_fdset:
+no_socket_pair:
   {
     g_free (newconn);
     return GST_RTSP_ESYS;
@@ -184,8 +204,6 @@ no_fdset:
  * forever. If @timeout contains a valid timeout, this function will return
  * #GST_RTSP_ETIMEOUT after the timeout expired.
  *
- * This function can be cancelled with gst_rtsp_connection_flush().
- *
  * Returns: #GST_RTSP_OK when a connection could be made.
  */
 GstRTSPResult
@@ -194,25 +212,25 @@ gst_rtsp_connection_connect (GstRTSPConnection * conn, GTimeVal * timeout)
   gint fd;
   struct sockaddr_in sa_in;
   struct hostent *hostinfo;
+  char **addrs;
   const gchar *ip;
+  gchar ipbuf[INET_ADDRSTRLEN];
   struct in_addr addr;
   gint ret;
   guint16 port;
   GstRTSPUrl *url;
-  GstClockTime to;
-  gint retval;
+  fd_set writefds;
+  fd_set readfds;
+  struct timeval tv, *tvp;
+  gint max_fd, retval;
 
 #ifdef G_OS_WIN32
-  unsigned long flags = 1;
-  struct in_addr *addrp;
-#else
-  char **addrs;
-  gchar ipbuf[INET_ADDRSTRLEN];
+  unsigned long flags;
 #endif /* G_OS_WIN32 */
 
   g_return_val_if_fail (conn != NULL, GST_RTSP_EINVAL);
   g_return_val_if_fail (conn->url != NULL, GST_RTSP_EINVAL);
-  g_return_val_if_fail (conn->fd.fd < 0, GST_RTSP_EINVAL);
+  g_return_val_if_fail (conn->fd < 0, GST_RTSP_EINVAL);
 
   url = conn->url;
 
@@ -226,15 +244,10 @@ gst_rtsp_connection_connect (GstRTSPConnection * conn, GTimeVal * timeout)
 
     if (hostinfo->h_addrtype != AF_INET)
       goto not_ip;              /* host not an IP host */
-#ifdef G_OS_WIN32
-    addrp = (struct in_addr *) hostinfo->h_addr_list[0];
-    /* this is not threadsafe */
-    ip = inet_ntoa (*addrp);
-#else
+
     addrs = hostinfo->h_addr_list;
     ip = inet_ntop (AF_INET, (struct in_addr *) addrs[0], ipbuf,
         sizeof (ipbuf));
-#endif /* G_OS_WIN32 */
   }
 
   /* get the port from the url */
@@ -256,46 +269,48 @@ gst_rtsp_connection_connect (GstRTSPConnection * conn, GTimeVal * timeout)
   ioctlsocket (fd, FIONBIO, &flags);
 #endif /* G_OS_WIN32 */
 
-  /* add the socket to our fdset */
-  conn->fd.fd = fd;
-  gst_poll_add_fd (conn->fdset, &conn->fd);
-
   /* we are going to connect ASYNC now */
   ret = connect (fd, (struct sockaddr *) &sa_in, sizeof (sa_in));
   if (ret == 0)
     goto done;
-  if (ERRNO_IS_NOT_EINPROGRESS)
+  if (errno != EINPROGRESS)
     goto sys_error;
 
   /* wait for connect to complete up to the specified timeout or until we got
    * interrupted. */
-  gst_poll_fd_ctl_write (conn->fdset, &conn->fd, TRUE);
+  FD_ZERO (&writefds);
+  FD_SET (fd, &writefds);
+  FD_ZERO (&readfds);
+  FD_SET (READ_SOCKET (conn), &readfds);
 
-  to = timeout ? GST_TIMEVAL_TO_TIME (*timeout) : GST_CLOCK_TIME_NONE;
+  if (timeout->tv_sec != 0 || timeout->tv_usec != 0) {
+    tv.tv_sec = timeout->tv_sec;
+    tv.tv_usec = timeout->tv_usec;
+    tvp = &tv;
+  } else {
+    tvp = NULL;
+  }
+
+  max_fd = MAX (fd, READ_SOCKET (conn));
 
   do {
-    retval = gst_poll_wait (conn->fdset, to);
-  } while (retval == -1 && errno == EINTR);
+    retval = select (max_fd + 1, &readfds, &writefds, NULL, tvp);
+  } while ((retval == -1 && errno == EINTR));
 
   if (retval == 0)
     goto timeout;
   else if (retval == -1)
     goto sys_error;
 
-  gst_poll_fd_ignored (conn->fdset, &conn->fd);
-
 done:
+  conn->fd = fd;
   conn->ip = g_strdup (ip);
 
   return GST_RTSP_OK;
 
 sys_error:
   {
-    if (conn->fd.fd >= 0) {
-      gst_poll_remove_fd (conn->fdset, &conn->fd);
-      conn->fd.fd = -1;
-    }
-    if (fd >= 0)
+    if (fd != -1)
       CLOSE_SOCKET (fd);
     return GST_RTSP_ESYS;
   }
@@ -309,12 +324,6 @@ not_ip:
   }
 timeout:
   {
-    if (conn->fd.fd >= 0) {
-      gst_poll_remove_fd (conn->fdset, &conn->fd);
-      conn->fd.fd = -1;
-    }
-    if (fd >= 0)
-      CLOSE_SOCKET (fd);
     return GST_RTSP_ETIMEOUT;
   }
 }
@@ -349,22 +358,10 @@ add_date_header (GstRTSPMessage * message)
 {
   GTimeVal tv;
   gchar date_string[100];
-  time_t t;
-
-#ifdef HAVE_GMTIME_R
-  struct tm tm_;
-#endif
 
   g_get_current_time (&tv);
-  t = (time_t) tv.tv_sec;
-
-#ifdef HAVE_GMTIME_R
   strftime (date_string, sizeof (date_string), "%a, %d %b %Y %H:%M:%S GMT",
-      gmtime_r (&t, &tm_));
-#else
-  strftime (date_string, sizeof (date_string), "%a, %d %b %Y %H:%M:%S GMT",
-      gmtime (&t));
-#endif
+      gmtime (&tv.tv_sec));
 
   gst_rtsp_message_add_header (message, GST_RTSP_HDR_DATE, date_string);
 }
@@ -380,7 +377,7 @@ add_date_header (GstRTSPMessage * message)
  * the specified @timeout. @timeout can be #NULL, in which case this function
  * might block forever.
  * 
- * This function can be cancelled with gst_rtsp_connection_flush().
+ * This function can be canceled with gst_rtsp_connection_flush().
  *
  * Returns: #GST_RTSP_OK on success.
  */
@@ -389,17 +386,29 @@ gst_rtsp_connection_write (GstRTSPConnection * conn, const guint8 * data,
     guint size, GTimeVal * timeout)
 {
   guint towrite;
+  fd_set writefds;
+  fd_set readfds;
+  int max_fd;
   gint retval;
-  GstClockTime to;
+  struct timeval tv, *tvp;
 
   g_return_val_if_fail (conn != NULL, GST_RTSP_EINVAL);
   g_return_val_if_fail (data != NULL || size == 0, GST_RTSP_EINVAL);
 
-  gst_poll_set_controllable (conn->fdset, TRUE);
-  gst_poll_fd_ctl_write (conn->fdset, &conn->fd, TRUE);
-  gst_poll_fd_ctl_read (conn->fdset, &conn->fd, FALSE);
+  FD_ZERO (&writefds);
+  FD_SET (conn->fd, &writefds);
+  FD_ZERO (&readfds);
+  FD_SET (READ_SOCKET (conn), &readfds);
 
-  to = timeout ? GST_TIMEVAL_TO_TIME (*timeout) : GST_CLOCK_TIME_NONE;
+  max_fd = MAX (conn->fd, READ_SOCKET (conn));
+
+  if (timeout) {
+    tv.tv_sec = timeout->tv_sec;
+    tv.tv_usec = timeout->tv_usec;
+    tvp = &tv;
+  } else {
+    tvp = NULL;
+  }
 
   towrite = size;
 
@@ -407,23 +416,34 @@ gst_rtsp_connection_write (GstRTSPConnection * conn, const guint8 * data,
     gint written;
 
     do {
-      retval = gst_poll_wait (conn->fdset, to);
-    } while (retval == -1 && errno == EINTR);
+      retval = select (max_fd + 1, &readfds, &writefds, NULL, tvp);
+    } while ((retval == -1 && errno == EINTR));
 
     if (retval == 0)
       goto timeout;
 
-    if (retval == -1) {
-      if (errno == EBUSY)
-        goto stopped;
-      else
-        goto select_error;
+    if (retval == -1)
+      goto select_error;
+
+    if (FD_ISSET (READ_SOCKET (conn), &readfds)) {
+      /* read all stop commands */
+      while (TRUE) {
+        gchar command;
+        int res;
+
+        READ_COMMAND (conn, command, res);
+        if (res <= 0) {
+          /* no more commands */
+          break;
+        }
+      }
+      goto stopped;
     }
 
     /* now we can write */
-    written = WRITE_SOCKET (conn->fd.fd, data, towrite);
+    written = write (conn->fd, data, towrite);
     if (written < 0) {
-      if (ERRNO_IS_NOT_EAGAIN && ERRNO_IS_NOT_EINTR)
+      if (errno != EAGAIN && errno != EINTR)
         goto write_error;
     } else {
       towrite -= written;
@@ -461,7 +481,7 @@ write_error:
  * the specified @timeout. @timeout can be #NULL, in which case this function
  * might block forever.
  * 
- * This function can be cancelled with gst_rtsp_connection_flush().
+ * This function can be canceled with gst_rtsp_connection_flush().
  *
  * Returns: #GST_RTSP_OK on success.
  */
@@ -530,7 +550,7 @@ gst_rtsp_connection_send (GstRTSPConnection * conn, GstRTSPMessage * message,
       break;
     }
     default:
-      g_return_val_if_reached (GST_RTSP_EINVAL);
+      g_assert_not_reached ();
       break;
   }
 
@@ -594,11 +614,11 @@ read_line (gint fd, gchar * buffer, guint size)
 
   idx = 0;
   while (TRUE) {
-    r = READ_SOCKET (fd, &c, 1);
+    r = read (fd, &c, 1);
     if (r == 0) {
       goto eof;
     } else if (r < 0) {
-      if (ERRNO_IS_NOT_EAGAIN && ERRNO_IS_NOT_EINTR)
+      if (errno != EAGAIN && errno != EINTR)
         goto read_error;
     } else {
       if (c == '\n')            /* end on \n */
@@ -767,30 +787,34 @@ no_column:
 }
 
 /**
- * gst_rtsp_connection_read_internal:
+ * gst_rtsp_connection_read:
  * @conn: a #GstRTSPConnection
  * @data: the data to read
  * @size: the size of @data
  * @timeout: a timeout value or #NULL
- * @allow_interrupt: can the pending read be interrupted
  *
  * Attempt to read @size bytes into @data from the connected @conn, blocking up to
  * the specified @timeout. @timeout can be #NULL, in which case this function
  * might block forever.
  * 
- * This function can be cancelled with gst_rtsp_connection_flush() only if
- * @allow_interrupt is set.
+ * This function can be canceled with gst_rtsp_connection_flush().
  *
  * Returns: #GST_RTSP_OK on success.
  */
-static GstRTSPResult
-gst_rtsp_connection_read_internal (GstRTSPConnection * conn, guint8 * data,
-    guint size, GTimeVal * timeout, gboolean allow_interrupt)
+GstRTSPResult
+gst_rtsp_connection_read (GstRTSPConnection * conn, guint8 * data, guint size,
+    GTimeVal * timeout)
 {
+  fd_set readfds;
   guint toread;
   gint retval;
-  GstClockTime to;
-  FIONREAD_TYPE avail;
+  struct timeval tv_timeout, *ptv_timeout = NULL;
+
+#ifndef G_OS_WIN32
+  gint avail;
+#else
+  gulong avail;
+#endif
 
   g_return_val_if_fail (conn != NULL, GST_RTSP_EINVAL);
   g_return_val_if_fail (data != NULL, GST_RTSP_EINVAL);
@@ -800,46 +824,62 @@ gst_rtsp_connection_read_internal (GstRTSPConnection * conn, guint8 * data,
 
   toread = size;
 
-  /* configure timeout if any */
-  to = timeout ? GST_TIMEVAL_TO_TIME (*timeout) : GST_CLOCK_TIME_NONE;
-
   /* if the call fails, just go in the select.. it should not fail. Else if
    * there is enough data to read, skip the select call al together.*/
-  if (IOCTL_SOCKET (conn->fd.fd, FIONREAD, &avail) < 0)
+  if (IOCTL_SOCKET (conn->fd, FIONREAD, &avail) < 0)
     avail = 0;
   else if (avail >= toread)
     goto do_read;
 
-  gst_poll_set_controllable (conn->fdset, allow_interrupt);
-  gst_poll_fd_ctl_write (conn->fdset, &conn->fd, FALSE);
-  gst_poll_fd_ctl_read (conn->fdset, &conn->fd, TRUE);
+  /* configure timeout if any */
+  if (timeout != NULL) {
+    tv_timeout.tv_sec = timeout->tv_sec;
+    tv_timeout.tv_usec = timeout->tv_usec;
+    ptv_timeout = &tv_timeout;
+  }
+
+  FD_ZERO (&readfds);
+  FD_SET (conn->fd, &readfds);
+  FD_SET (READ_SOCKET (conn), &readfds);
 
   while (toread > 0) {
     gint bytes;
 
     do {
-      retval = gst_poll_wait (conn->fdset, to);
-    } while (retval == -1 && errno == EINTR);
+      retval = select (FD_SETSIZE, &readfds, NULL, NULL, ptv_timeout);
+    } while ((retval == -1 && errno == EINTR));
 
-    if (retval == -1) {
-      if (errno == EBUSY)
-        goto stopped;
-      else
-        goto select_error;
-    }
+    if (retval == -1)
+      goto select_error;
 
     /* check for timeout */
     if (retval == 0)
       goto select_timeout;
 
+    if (FD_ISSET (READ_SOCKET (conn), &readfds)) {
+      /* read all stop commands */
+      while (TRUE) {
+        gchar command;
+        int res;
+
+        READ_COMMAND (conn, command, res);
+        if (res <= 0) {
+          /* no more commands */
+          break;
+        }
+      }
+      goto stopped;
+    }
+
   do_read:
     /* if we get here there is activity on the real fd since the select
      * completed and the control socket was not readable. */
-    bytes = READ_SOCKET (conn->fd.fd, data, toread);
+    bytes = read (conn->fd, data, toread);
+
     if (bytes == 0) {
       goto eof;
     } else if (bytes < 0) {
-      if (ERRNO_IS_NOT_EAGAIN && ERRNO_IS_NOT_EINTR)
+      if (errno != EAGAIN && errno != EINTR)
         goto read_error;
     } else {
       toread -= bytes;
@@ -871,29 +911,6 @@ read_error:
   }
 }
 
-/**
- * gst_rtsp_connection_read:
- * @conn: a #GstRTSPConnection
- * @data: the data to read
- * @size: the size of @data
- * @timeout: a timeout value or #NULL
- *
- * Attempt to read @size bytes into @data from the connected @conn, blocking up to
- * the specified @timeout. @timeout can be #NULL, in which case this function
- * might block forever.
- *
- * This function can be cancelled with gst_rtsp_connection_flush().
- *
- * Returns: #GST_RTSP_OK on success.
- */
-GstRTSPResult
-gst_rtsp_connection_read (GstRTSPConnection * conn, guint8 * data, guint size,
-    GTimeVal * timeout)
-{
-  return gst_rtsp_connection_read_internal (conn, data, size, timeout, TRUE);
-}
-
-
 static GstRTSPResult
 read_body (GstRTSPConnection * conn, glong content_length, GstRTSPMessage * msg,
     GTimeVal * timeout)
@@ -910,8 +927,8 @@ read_body (GstRTSPConnection * conn, glong content_length, GstRTSPMessage * msg,
   body = g_malloc (content_length + 1);
   body[content_length] = '\0';
 
-  GST_RTSP_CHECK (gst_rtsp_connection_read_internal (conn, body, content_length,
-          timeout, FALSE), read_error);
+  GST_RTSP_CHECK (gst_rtsp_connection_read (conn, body, content_length,
+          timeout), read_error);
 
   content_length += 1;
 
@@ -938,7 +955,7 @@ read_error:
  * the specified @timeout. @timeout can be #NULL, in which case this function
  * might block forever.
  * 
- * This function can be cancelled with gst_rtsp_connection_flush().
+ * This function can be canceled with gst_rtsp_connection_flush().
  *
  * Returns: #GST_RTSP_OK on success.
  */
@@ -965,9 +982,8 @@ gst_rtsp_connection_receive (GstRTSPConnection * conn, GstRTSPMessage * message,
     guint8 c;
 
     /* read first character, this identifies data messages */
-    /* This is the only read() that we allow to be interrupted */
-    GST_RTSP_CHECK (gst_rtsp_connection_read_internal (conn, &c, 1, timeout,
-            TRUE), read_error);
+    GST_RTSP_CHECK (gst_rtsp_connection_read (conn, &c, 1, timeout),
+        read_error);
 
     /* check for data packet, first character is $ */
     if (c == '$') {
@@ -976,15 +992,15 @@ gst_rtsp_connection_receive (GstRTSPConnection * conn, GstRTSPMessage * message,
       /* data packets are $<1 byte channel><2 bytes length,BE><data bytes> */
 
       /* read channel, which is the next char */
-      GST_RTSP_CHECK (gst_rtsp_connection_read_internal (conn, &c, 1, timeout,
-              FALSE), read_error);
+      GST_RTSP_CHECK (gst_rtsp_connection_read (conn, &c, 1, timeout),
+          read_error);
 
       /* now we create a data message */
       gst_rtsp_message_init_data (message, c);
 
       /* next two bytes are the length of the data */
-      GST_RTSP_CHECK (gst_rtsp_connection_read_internal (conn,
-              (guint8 *) & size, 2, timeout, FALSE), read_error);
+      GST_RTSP_CHECK (gst_rtsp_connection_read (conn, (guint8 *) & size, 2,
+              timeout), read_error);
 
       size = GUINT16_FROM_BE (size);
 
@@ -1005,7 +1021,7 @@ gst_rtsp_connection_receive (GstRTSPConnection * conn, GstRTSPMessage * message,
         break;
 
       /* read lines */
-      GST_RTSP_CHECK (read_line (conn->fd.fd, buffer + offset,
+      GST_RTSP_CHECK (read_line (conn->fd, buffer + offset,
               sizeof (buffer) - offset), read_error);
 
       if (buffer[0] == '\0')
@@ -1100,13 +1116,12 @@ gst_rtsp_connection_close (GstRTSPConnection * conn)
   g_free (conn->ip);
   conn->ip = NULL;
 
-  if (conn->fd.fd != -1) {
-    gst_poll_remove_fd (conn->fdset, &conn->fd);
-    res = CLOSE_SOCKET (conn->fd.fd);
-    conn->fd.fd = -1;
+  if (conn->fd != -1) {
+    res = CLOSE_SOCKET (conn->fd);
 #ifdef G_OS_WIN32
     WSACleanup ();
 #endif
+    conn->fd = -1;
     if (res != 0)
       goto sys_error;
   }
@@ -1134,11 +1149,10 @@ gst_rtsp_connection_free (GstRTSPConnection * conn)
 
   g_return_val_if_fail (conn != NULL, GST_RTSP_EINVAL);
 
-  res = gst_rtsp_connection_close (conn);
-  gst_poll_free (conn->fdset);
 #ifdef G_OS_WIN32
   WSACleanup ();
 #endif
+  res = gst_rtsp_connection_close (conn);
   g_timer_destroy (conn->timer);
   g_free (conn->username);
   g_free (conn->passwd);
@@ -1147,88 +1161,6 @@ gst_rtsp_connection_free (GstRTSPConnection * conn)
   return res;
 }
 
-/**
- * gst_rtsp_connection_poll:
- * @conn: a #GstRTSPConnection
- * @events: a bitmask of #GstRTSPEvent flags to check
- * @revents: location for result flags 
- * @timeout: a timeout
- *
- * Wait up to the specified @timeout for the connection to become available for
- * at least one of the operations specified in @events. When the function returns
- * with #GST_RTSP_OK, @revents will contain a bitmask of available operations on
- * @conn.
- *
- * @timeout can be #NULL, in which case this function might block forever.
- *
- * This function can be cancelled with gst_rtsp_connection_flush().
- * 
- * Returns: #GST_RTSP_OK on success.
- *
- * Since: 0.10.15
- */
-GstRTSPResult
-gst_rtsp_connection_poll (GstRTSPConnection * conn, GstRTSPEvent events,
-    GstRTSPEvent * revents, GTimeVal * timeout)
-{
-  GstClockTime to;
-  gint retval;
-
-  g_return_val_if_fail (conn != NULL, GST_RTSP_EINVAL);
-  g_return_val_if_fail (events != 0, GST_RTSP_EINVAL);
-  g_return_val_if_fail (revents != NULL, GST_RTSP_EINVAL);
-
-  gst_poll_set_controllable (conn->fdset, TRUE);
-
-  /* add fd to writer set when asked to */
-  gst_poll_fd_ctl_write (conn->fdset, &conn->fd, events & GST_RTSP_EV_WRITE);
-
-  /* add fd to reader set when asked to */
-  gst_poll_fd_ctl_read (conn->fdset, &conn->fd, events & GST_RTSP_EV_READ);
-
-  /* configure timeout if any */
-  to = timeout ? GST_TIMEVAL_TO_TIME (*timeout) : GST_CLOCK_TIME_NONE;
-
-  do {
-    retval = gst_poll_wait (conn->fdset, to);
-  } while (retval == -1 && errno == EINTR);
-
-  if (retval == 0)
-    goto select_timeout;
-
-  if (retval == -1) {
-    if (errno == EBUSY)
-      goto stopped;
-    else
-      goto select_error;
-  }
-
-  *revents = 0;
-  if (events & GST_RTSP_EV_READ) {
-    if (gst_poll_fd_can_read (conn->fdset, &conn->fd))
-      *revents |= GST_RTSP_EV_READ;
-  }
-  if (events & GST_RTSP_EV_WRITE) {
-    if (gst_poll_fd_can_write (conn->fdset, &conn->fd))
-      *revents |= GST_RTSP_EV_WRITE;
-  }
-  return GST_RTSP_OK;
-
-  /* ERRORS */
-select_timeout:
-  {
-    return GST_RTSP_ETIMEOUT;
-  }
-select_error:
-  {
-    return GST_RTSP_ESYS;
-  }
-stopped:
-  {
-    return GST_RTSP_EINTR;
-  }
-}
-
 /**
  * gst_rtsp_connection_next_timeout:
  * @conn: a #GstRTSPConnection
@@ -1296,8 +1228,20 @@ gst_rtsp_connection_flush (GstRTSPConnection * conn, gboolean flush)
 {
   g_return_val_if_fail (conn != NULL, GST_RTSP_EINVAL);
 
-  gst_poll_set_flushing (conn->fdset, flush);
+  if (flush) {
+    SEND_COMMAND (conn, CONTROL_STOP);
+  } else {
+    while (TRUE) {
+      gchar command;
+      int res;
 
+      READ_COMMAND (conn, command, res);
+      if (res <= 0) {
+        /* no more commands */
+        break;
+      }
+    }
+  }
   return GST_RTSP_OK;
 }
 
