@@ -47,114 +47,71 @@
  * <refsect2>
  * <para>
  * The GstSDPMessage helper functions makes it easy to parse and create SDP
- * messages.
+ * messages. 
  * </para>
  * </refsect2>
  *
  * Last reviewed on 2007-07-24 (0.10.14)
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
 
 #include <stdlib.h>
 #include <string.h>
 
-#ifdef HAVE_SYS_TYPES_H
-#include <sys/types.h>
-#endif
-
-#include <glib.h>               /* for G_OS_WIN32 */
-#include <gst/gstinfo.h>        /* For GST_STR_NULL */
-
-#ifdef G_OS_WIN32
-/* ws2_32.dll has getaddrinfo and freeaddrinfo on Windows XP and later.
- * minwg32 headers check WINVER before allowing the use of these */
-#ifndef WINVER
-#define WINVER 0x0501
-#endif
-#ifdef _MSC_VER
-#include <Winsock2.h>
-#endif
-#include <ws2tcpip.h>
-#else
-#include <sys/socket.h>
-#include <netdb.h>
-#include <netinet/in.h>
-#endif
-
 #include "gstsdpmessage.h"
 
 /* FIXME, is currently allocated on the stack */
-#define MAX_LINE_LEN    1024 * 16
+#define MAX_LINE_LEN	1024 * 16
 
-#define FREE_STRING(field)              g_free (field); (field) = NULL
-#define REPLACE_STRING(field, val)      FREE_STRING(field); (field) = g_strdup (val)
-
-#define INIT_ARRAY(field, type, init_func)              \
-G_STMT_START {                                          \
-  if (field) {                                          \
-    guint i;                                            \
-    for(i = 0; i < (field)->len; i++)                   \
-      init_func (&g_array_index ((field), type, i));    \
-    g_array_set_size ((field), 0);                      \
-  }                                                     \
-  else                                                  \
-    (field) = g_array_new (FALSE, TRUE, sizeof (type)); \
+#define FREE_STRING(field)      g_free ((field)); (field) = NULL;
+#define FREE_ARRAY(field)       \
+G_STMT_START {                  \
+  if (field)                    \
+    g_array_free (field, TRUE); \
+  field = NULL;                 \
 } G_STMT_END
+#define REPLACE_STRING(field,val)       FREE_STRING(field);field=g_strdup (val);
 
-#define FREE_ARRAY(field)         \
-G_STMT_START {                    \
-  if (field)                      \
-    g_array_free ((field), TRUE); \
-  (field) = NULL;                 \
+#define INIT_ARRAY(field,type,init_func)  		\
+G_STMT_START {                   			\
+  if (field) {                   			\
+    guint i;			 			\
+    for(i=0; i<field->len; i++)				\
+      init_func (&g_array_index(field, type, i));	\
+    g_array_set_size (field,0); 			\
+  }				 			\
+  else                          			\
+    field = g_array_new (FALSE, TRUE, sizeof(type));    \
 } G_STMT_END
 
-#define INIT_PTR_ARRAY(field, type, init_func)          \
-G_STMT_START {                                          \
-  if (field) {                                          \
-    guint i;                                            \
-    for(i = 0; i < (field)->len; i++)                   \
-      init_func (g_array_index ((field), type, i));     \
-    g_array_set_size ((field), 0);                      \
-  }                                                     \
-  else                                                  \
-    (field) = g_array_new (FALSE, TRUE, sizeof (type)); \
-} G_STMT_END
-
-#define FREE_PTR_ARRAY(field) FREE_ARRAY(field)
-
 #define DEFINE_STRING_SETTER(field)                                     \
-GstSDPResult gst_sdp_message_set_##field (GstSDPMessage *msg, const gchar *val) { \
+GstSDPResult gst_sdp_message_set_##field (GstSDPMessage *msg, const gchar *val) {      \
   g_free (msg->field);                                                  \
   msg->field = g_strdup (val);                                          \
-  return GST_SDP_OK;                                                    \
+  return GST_SDP_OK;                                                       \
 }
 #define DEFINE_STRING_GETTER(field)                                     \
-const gchar* gst_sdp_message_get_##field (const GstSDPMessage *msg) {   \
+const gchar* gst_sdp_message_get_##field (GstSDPMessage *msg) {                       \
   return msg->field;                                                    \
 }
 
 #define DEFINE_ARRAY_LEN(field)                                         \
-guint gst_sdp_message_##field##_len (const GstSDPMessage *msg) {        \
-  return msg->field->len;                                               \
+gint gst_sdp_message_##field##_len (GstSDPMessage *msg) {                      \
+  return ((msg)->field->len);                                           \
 }
-#define DEFINE_ARRAY_GETTER(method, field, type)                        \
-type * gst_sdp_message_get_##method (const GstSDPMessage *msg, guint idx) {  \
-  return &g_array_index (msg->field, type, idx);                        \
+#define DEFINE_ARRAY_GETTER(method,field,type)                          \
+type gst_sdp_message_get_##method (GstSDPMessage *msg, guint idx) {            \
+  return g_array_index ((msg)->field, type, idx);                       \
 }
-
-#define DEFINE_PTR_ARRAY_LEN(field) DEFINE_ARRAY_LEN(field)
-#define DEFINE_PTR_ARRAY_GETTER(method, field, type)                    \
-type gst_sdp_message_get_##method (const GstSDPMessage *msg, guint idx) {    \
-  return g_array_index (msg->field, type, idx);                         \
+#define DEFINE_ARRAY_P_GETTER(method,field,type)                        \
+type * gst_sdp_message_get_##method (GstSDPMessage *msg, guint idx) {          \
+  return &g_array_index ((msg)->field, type, idx);                      \
 }
-#define DEFINE_PTR_ARRAY_ADDER(method, field, type, dup_method)         \
-GstSDPResult gst_sdp_message_add_##method (GstSDPMessage *msg, type val) {   \
-  type v = dup_method (val);                                            \
-  g_array_append_val (msg->field, v);                                   \
-  return GST_SDP_OK;                                                    \
+#define DEFINE_ARRAY_ADDER(method,field,type,dup_method)                \
+GstSDPResult gst_sdp_message_add_##method (GstSDPMessage *msg, type val) {       \
+  type v = dup_method(val);                                             \
+  g_array_append_val((msg)->field, v);                                  \
+  return GST_SDP_OK;                                                       \
 }
 
 static void
@@ -186,12 +143,11 @@ gst_sdp_bandwidth_init (GstSDPBandwidth * bandwidth)
 }
 
 static void
-gst_sdp_time_init (GstSDPTime * t)
+gst_sdp_time_init (GstSDPTime * time)
 {
-  FREE_STRING (t->start);
-  FREE_STRING (t->stop);
-  INIT_PTR_ARRAY (t->repeat, gchar *, g_free);
-  FREE_PTR_ARRAY (t->repeat);
+  FREE_STRING (time->start);
+  FREE_STRING (time->stop);
+  time->n_repeat = 0;
 }
 
 static void
@@ -260,8 +216,8 @@ gst_sdp_message_init (GstSDPMessage * msg)
   FREE_STRING (msg->session_name);
   FREE_STRING (msg->information);
   FREE_STRING (msg->uri);
-  INIT_PTR_ARRAY (msg->emails, gchar *, g_free);
-  INIT_PTR_ARRAY (msg->phones, gchar *, g_free);
+  INIT_ARRAY (msg->emails, gchar *, g_free);
+  INIT_ARRAY (msg->phones, gchar *, g_free);
   gst_sdp_connection_init (&msg->connection);
   INIT_ARRAY (msg->bandwidths, GstSDPBandwidth, gst_sdp_bandwidth_init);
   INIT_ARRAY (msg->times, GstSDPTime, gst_sdp_time_init);
@@ -290,8 +246,8 @@ gst_sdp_message_uninit (GstSDPMessage * msg)
 
   gst_sdp_message_init (msg);
 
-  FREE_PTR_ARRAY (msg->emails);
-  FREE_PTR_ARRAY (msg->phones);
+  FREE_ARRAY (msg->emails);
+  FREE_ARRAY (msg->phones);
   FREE_ARRAY (msg->bandwidths);
   FREE_ARRAY (msg->times);
   FREE_ARRAY (msg->zones);
@@ -322,170 +278,6 @@ gst_sdp_message_free (GstSDPMessage * msg)
   return GST_SDP_OK;
 }
 
-static gboolean
-is_multicast_address (const gchar * host_name, guint * family)
-{
-  struct addrinfo hints;
-  struct addrinfo *ai;
-  struct addrinfo *res;
-  gboolean ret = FALSE;
-
-  memset (&hints, 0, sizeof (hints));
-  hints.ai_socktype = SOCK_DGRAM;
-
-  g_return_val_if_fail (host_name, FALSE);
-
-  if (getaddrinfo (host_name, NULL, &hints, &res) < 0)
-    return FALSE;
-
-  for (ai = res; !ret && ai; ai = ai->ai_next) {
-    if (ai->ai_family == AF_INET)
-      ret =
-          IN_MULTICAST (ntohl (((struct sockaddr_in *) ai->ai_addr)->sin_addr.
-              s_addr));
-    else
-      ret =
-          IN6_IS_ADDR_MULTICAST (&((struct sockaddr_in6 *) ai->ai_addr)->
-          sin6_addr);
-    if (ret && family)
-      *family = ai->ai_family;
-  }
-
-  freeaddrinfo (res);
-
-  return ret;
-}
-
-/**
- * gst_sdp_message_as_text:
- * @msg: a #GstSDPMessage
- *
- * Convert the contents of @msg to a text string.
- *
- * Returns: A dynamically allocated string representing the SDP description.
- */
-gchar *
-gst_sdp_message_as_text (const GstSDPMessage * msg)
-{
-  /* change all vars so they match rfc? */
-  GString *lines;
-  guint i;
-
-  g_return_val_if_fail (msg != NULL, NULL);
-
-  lines = g_string_new ("");
-
-  if (msg->version)
-    g_string_append_printf (lines, "v=%s\r\n", msg->version);
-
-  if (msg->origin.sess_id && msg->origin.sess_version && msg->origin.nettype &&
-      msg->origin.addrtype && msg->origin.addr)
-    g_string_append_printf (lines, "o=%s %s %s %s %s %s\r\n",
-        msg->origin.username ? msg->origin.username : "-", msg->origin.sess_id,
-        msg->origin.sess_version, msg->origin.nettype, msg->origin.addrtype,
-        msg->origin.addr);
-
-  if (msg->session_name)
-    g_string_append_printf (lines, "s=%s\r\n", msg->session_name);
-
-  if (msg->information)
-    g_string_append_printf (lines, "i=%s\r\n", msg->information);
-
-  if (msg->uri)
-    g_string_append_printf (lines, "u=%s\r\n", msg->uri);
-
-  for (i = 0; i < gst_sdp_message_emails_len (msg); i++)
-    g_string_append_printf (lines, "e=%s\r\n",
-        gst_sdp_message_get_email (msg, i));
-
-  for (i = 0; i < gst_sdp_message_phones_len (msg); i++)
-    g_string_append_printf (lines, "p=%s\r\n",
-        gst_sdp_message_get_phone (msg, i));
-
-  if (gst_sdp_message_emails_len (msg) == 0 &&
-      gst_sdp_message_phones_len (msg) == 0)
-    g_string_append_printf (lines, "e=NONE\r\n");
-
-  if (msg->connection.nettype && msg->connection.addrtype &&
-      msg->connection.address) {
-    guint family;
-
-    g_string_append_printf (lines, "c=%s %s %s", msg->connection.nettype,
-        msg->connection.addrtype, msg->connection.address);
-    if (is_multicast_address (msg->connection.address, &family)) {
-      if (family == AF_INET)
-        g_string_append_printf (lines, "/%u", msg->connection.ttl);
-      if (msg->connection.addr_number > 1)
-        g_string_append_printf (lines, "/%u", msg->connection.addr_number);
-    }
-    g_string_append_printf (lines, "\r\n");
-  }
-
-  for (i = 0; i < gst_sdp_message_bandwidths_len (msg); i++) {
-    const GstSDPBandwidth *bandwidth = gst_sdp_message_get_bandwidth (msg, i);
-
-    g_string_append_printf (lines, "b=%s:%u\r\n", bandwidth->bwtype,
-        bandwidth->bandwidth);
-  }
-
-  for (i = 0; i < gst_sdp_message_times_len (msg); i++) {
-    const GstSDPTime *times = gst_sdp_message_get_time (msg, i);
-
-    g_string_append_printf (lines, "t=%s %s\r\n", times->start, times->stop);
-
-    if (times->repeat != NULL) {
-      guint j;
-
-      g_string_append_printf (lines, "r=%s",
-          g_array_index (times->repeat, gchar *, 0));
-      for (j = 1; j < times->repeat->len; j++)
-        g_string_append_printf (lines, " %s",
-            g_array_index (times->repeat, gchar *, j));
-      g_string_append_printf (lines, "\r\n");
-    }
-  }
-
-  if (gst_sdp_message_zones_len (msg) > 0) {
-    const GstSDPZone *zone = gst_sdp_message_get_zone (msg, 0);
-
-    g_string_append_printf (lines, "z=%s %s", zone->time, zone->typed_time);
-    for (i = 1; i < gst_sdp_message_zones_len (msg); i++) {
-      zone = gst_sdp_message_get_zone (msg, i);
-      g_string_append_printf (lines, " %s %s", zone->time, zone->typed_time);
-    }
-    g_string_append_printf (lines, "\r\n");
-  }
-
-  if (msg->key.type) {
-    g_string_append_printf (lines, "k=%s", msg->key.type);
-    if (msg->key.data)
-      g_string_append_printf (lines, ":%s", msg->key.data);
-    g_string_append_printf (lines, "\r\n");
-  }
-
-  for (i = 0; i < gst_sdp_message_attributes_len (msg); i++) {
-    const GstSDPAttribute *attr = gst_sdp_message_get_attribute (msg, i);
-
-    if (attr->key) {
-      g_string_append_printf (lines, "a=%s", attr->key);
-      if (attr->value)
-        g_string_append_printf (lines, ":%s", attr->value);
-      g_string_append_printf (lines, "\r\n");
-    }
-  }
-
-  for (i = 0; i < gst_sdp_message_medias_len (msg); i++) {
-    const GstSDPMedia *media = gst_sdp_message_get_media (msg, i);
-    gchar *sdp_media_str;
-
-    sdp_media_str = gst_sdp_media_as_text (media);
-    g_string_append_printf (lines, "%s", sdp_media_str);
-    g_free (sdp_media_str);
-  }
-
-  return g_string_free (lines, FALSE);
-}
-
 /**
  * gst_sdp_message_set_version:
  * @msg: a #GstSDPMessage
@@ -544,7 +336,7 @@ gst_sdp_message_set_origin (GstSDPMessage * msg, const gchar * username,
  * Returns: a #GstSDPOrigin. The result remains valid as long as @msg is valid.
  */
 const GstSDPOrigin *
-gst_sdp_message_get_origin (const GstSDPMessage * msg)
+gst_sdp_message_get_origin (GstSDPMessage * msg)
 {
   return &msg->origin;
 }
@@ -623,9 +415,9 @@ DEFINE_ARRAY_LEN (emails);
  *
  * Get the email with number @idx from @msg.
  *
- * Returns: the email at position @idx.
+ * Returns: the email at position @idx. 
  */
-DEFINE_PTR_ARRAY_GETTER (email, emails, const gchar *);
+DEFINE_ARRAY_GETTER (email, emails, const gchar *);
 
 /**
  * gst_sdp_message_add_email:
@@ -636,7 +428,7 @@ DEFINE_PTR_ARRAY_GETTER (email, emails, const gchar *);
  *
  * Returns: a #GstSDPResult.
  */
-DEFINE_PTR_ARRAY_ADDER (email, emails, const gchar *, g_strdup);
+DEFINE_ARRAY_ADDER (email, emails, const gchar *, g_strdup);
 
 /**
  * gst_sdp_message_phones_len:
@@ -654,9 +446,9 @@ DEFINE_ARRAY_LEN (phones);
  *
  * Get the phone with number @idx from @msg.
  *
- * Returns: the phone at position @idx.
+ * Returns: the phone at position @idx. 
  */
-DEFINE_PTR_ARRAY_GETTER (phone, phones, const gchar *);
+DEFINE_ARRAY_GETTER (phone, phones, const gchar *);
 
 /**
  * gst_sdp_message_add_phone:
@@ -667,7 +459,7 @@ DEFINE_PTR_ARRAY_GETTER (phone, phones, const gchar *);
  *
  * Returns: a #GstSDPResult.
  */
-DEFINE_PTR_ARRAY_ADDER (phone, phones, const gchar *, g_strdup);
+DEFINE_ARRAY_ADDER (phone, phones, const gchar *, g_strdup);
 
 /**
  * gst_sdp_message_set_connection:
@@ -685,7 +477,7 @@ DEFINE_PTR_ARRAY_ADDER (phone, phones, const gchar *, g_strdup);
  */
 GstSDPResult
 gst_sdp_message_set_connection (GstSDPMessage * msg, const gchar * nettype,
-    const gchar * addrtype, const gchar * address, guint ttl, guint addr_number)
+    const gchar * addrtype, const gchar * address, gint ttl, gint addr_number)
 {
   REPLACE_STRING (msg->connection.nettype, nettype);
   REPLACE_STRING (msg->connection.addrtype, addrtype);
@@ -705,7 +497,7 @@ gst_sdp_message_set_connection (GstSDPMessage * msg, const gchar * nettype,
  * Returns: a #GstSDPConnection. The result remains valid as long as @msg is valid.
  */
 const GstSDPConnection *
-gst_sdp_message_get_connection (const GstSDPMessage * msg)
+gst_sdp_message_get_connection (GstSDPMessage * msg)
 {
   return &msg->connection;
 }
@@ -728,22 +520,21 @@ DEFINE_ARRAY_LEN (bandwidths);
  *
  * Returns: a #GstSDPBandwidth.
  */
-DEFINE_ARRAY_GETTER (bandwidth, bandwidths, const GstSDPBandwidth);
+DEFINE_ARRAY_P_GETTER (bandwidth, bandwidths, const GstSDPBandwidth);
 
 /**
  * gst_sdp_message_add_bandwidth:
  * @msg: a #GstSDPMessage
- * @bwtype: the bandwidth modifier type
+ * @bwtype: the bandwidth modifier type 
  * @bandwidth: the bandwidth in kilobits per second
  *
  * Add the specified bandwidth information to @msg.
  *
  * Returns: a #GstSDPResult.
  */
-
 GstSDPResult
 gst_sdp_message_add_bandwidth (GstSDPMessage * msg, const gchar * bwtype,
-    guint bandwidth)
+    gint bandwidth)
 {
   GstSDPBandwidth bw;
 
@@ -774,38 +565,20 @@ DEFINE_ARRAY_LEN (times);
  *
  * Returns: a #GstSDPTime.
  */
-DEFINE_ARRAY_GETTER (time, times, const GstSDPTime);
+DEFINE_ARRAY_P_GETTER (time, times, const GstSDPTime);
 
 /**
  * gst_sdp_message_add_time:
  * @msg: a #GstSDPMessage
- * @start: the start time
- * @stop: the stop time
- * @repeat: the repeat times
+ * @time: a time information
  *
- * Add time information @start and @stop to @msg.
+ * Add time information @time to @msg.
  *
  * Returns: a #GstSDPResult.
  */
 GstSDPResult
-gst_sdp_message_add_time (GstSDPMessage * msg, const gchar * start,
-    const gchar * stop, const gchar ** repeat)
+gst_sdp_message_add_time (GstSDPMessage * msg, const gchar * time)
 {
-  GstSDPTime times;
-
-  times.start = g_strdup (start);
-  times.stop = g_strdup (stop);
-  if (repeat) {
-    times.repeat = g_array_new (FALSE, TRUE, sizeof (gchar *));
-    for (; *repeat; repeat++) {
-      gchar *r = g_strdup (*repeat);
-
-      g_array_append_val (times.repeat, r);
-    }
-  } else
-    times.repeat = NULL;
-  g_array_append_val (msg->times, times);
-
   return GST_SDP_OK;
 }
 
@@ -827,25 +600,25 @@ DEFINE_ARRAY_LEN (zones);
  *
  * Returns: a #GstSDPZone.
  */
-DEFINE_ARRAY_GETTER (zone, zones, const GstSDPZone);
+DEFINE_ARRAY_P_GETTER (zone, zones, const GstSDPZone);
 
 /**
  * gst_sdp_message_add_zone:
  * @msg: a #GstSDPMessage
- * @adj_time: the NTP time that a time zone adjustment happens
- * @typed_time: the offset from the time when the session was first scheduled
+ * @time: the NTP time that a time zone adjustment happens
+ * @typed_time: the offset from the time when the session was first scheduled 
  *
  * Add time zone information to @msg.
  *
  * Returns: a #GstSDPResult.
  */
 GstSDPResult
-gst_sdp_message_add_zone (GstSDPMessage * msg, const gchar * adj_time,
+gst_sdp_message_add_zone (GstSDPMessage * msg, const gchar * time,
     const gchar * typed_time)
 {
   GstSDPZone zone;
 
-  zone.time = g_strdup (adj_time);
+  zone.time = g_strdup (time);
   zone.typed_time = g_strdup (typed_time);
 
   g_array_append_val (msg->zones, zone);
@@ -882,7 +655,7 @@ gst_sdp_message_set_key (GstSDPMessage * msg, const gchar * type,
  * Returns: a #GstSDPKey.
  */
 const GstSDPKey *
-gst_sdp_message_get_key (const GstSDPMessage * msg)
+gst_sdp_message_get_key (GstSDPMessage * msg)
 {
   return &msg->key;
 }
@@ -896,7 +669,6 @@ gst_sdp_message_get_key (const GstSDPMessage * msg)
  * Returns: the number of attributes in @msg.
  */
 DEFINE_ARRAY_LEN (attributes);
-
 /**
  * gst_sdp_message_get_attribute:
  * @msg: a #GstSDPMessage
@@ -906,8 +678,7 @@ DEFINE_ARRAY_LEN (attributes);
  *
  * Returns: the #GstSDPAttribute at position @idx.
  */
-DEFINE_ARRAY_GETTER (attribute, attributes, const GstSDPAttribute);
-
+DEFINE_ARRAY_P_GETTER (attribute, attributes, const GstSDPAttribute);
 /**
  * gst_sdp_message_get_attribute_val_n:
  * @msg: a #GstSDPMessage
@@ -919,8 +690,8 @@ DEFINE_ARRAY_GETTER (attribute, attributes, const GstSDPAttribute);
  * Returns: the attribute value of the @nth attribute with @key.
  */
 const gchar *
-gst_sdp_message_get_attribute_val_n (const GstSDPMessage * msg,
-    const gchar * key, guint nth)
+gst_sdp_message_get_attribute_val_n (GstSDPMessage * msg, const gchar * key,
+    guint nth)
 {
   guint i;
 
@@ -948,7 +719,7 @@ gst_sdp_message_get_attribute_val_n (const GstSDPMessage * msg,
  * Returns: the attribute value of the first attribute with @key.
  */
 const gchar *
-gst_sdp_message_get_attribute_val (const GstSDPMessage * msg, const gchar * key)
+gst_sdp_message_get_attribute_val (GstSDPMessage * msg, const gchar * key)
 {
   return gst_sdp_message_get_attribute_val_n (msg, key, 0);
 }
@@ -995,7 +766,7 @@ DEFINE_ARRAY_LEN (medias);
  *
  * Returns: a #GstSDPMedia.
  */
-DEFINE_ARRAY_GETTER (media, medias, const GstSDPMedia);
+DEFINE_ARRAY_P_GETTER (media, medias, const GstSDPMedia);
 
 /**
  * gst_sdp_message_add_media:
@@ -1011,7 +782,7 @@ DEFINE_ARRAY_GETTER (media, medias, const GstSDPMedia);
 GstSDPResult
 gst_sdp_message_add_media (GstSDPMessage * msg, GstSDPMedia * media)
 {
-  guint len;
+  gint len;
   GstSDPMedia *nmedia;
 
   len = msg->medias->len;
@@ -1070,7 +841,7 @@ gst_sdp_media_init (GstSDPMedia * media)
   media->port = 0;
   media->num_ports = 0;
   FREE_STRING (media->proto);
-  INIT_PTR_ARRAY (media->fmts, gchar *, g_free);
+  INIT_ARRAY (media->fmts, gchar *, g_free);
   FREE_STRING (media->information);
   INIT_ARRAY (media->connections, GstSDPConnection, gst_sdp_connection_init);
   INIT_ARRAY (media->bandwidths, GstSDPBandwidth, gst_sdp_bandwidth_init);
@@ -1096,7 +867,7 @@ gst_sdp_media_uninit (GstSDPMedia * media)
   g_return_val_if_fail (media != NULL, GST_SDP_EINVAL);
 
   gst_sdp_media_init (media);
-  FREE_PTR_ARRAY (media->fmts);
+  FREE_ARRAY (media->fmts);
   FREE_ARRAY (media->connections);
   FREE_ARRAY (media->bandwidths);
   FREE_ARRAY (media->attributes);
@@ -1125,87 +896,6 @@ gst_sdp_media_free (GstSDPMedia * media)
   return GST_SDP_OK;
 }
 
-/**
- * gst_sdp_media_as_text:
- * @media: a #GstSDPMedia
- *
- * Convert the contents of @media to a text string.
- *
- * Returns: A dynamically allocated string representing the media.
- */
-gchar *
-gst_sdp_media_as_text (const GstSDPMedia * media)
-{
-  GString *lines;
-  guint i;
-
-  g_return_val_if_fail (media != NULL, NULL);
-
-  lines = g_string_new ("");
-
-  if (media->media)
-    g_string_append_printf (lines, "m=%s", media->media);
-
-  g_string_append_printf (lines, " %u", media->port);
-
-  if (media->num_ports > 1)
-    g_string_append_printf (lines, "/%u", media->num_ports);
-
-  g_string_append_printf (lines, " %s", media->proto);
-
-  for (i = 0; i < gst_sdp_media_formats_len (media); i++)
-    g_string_append_printf (lines, " %s", gst_sdp_media_get_format (media, i));
-  g_string_append_printf (lines, "\r\n");
-
-  if (media->information)
-    g_string_append_printf (lines, "i=%s", media->information);
-
-  for (i = 0; i < gst_sdp_media_connections_len (media); i++) {
-    const GstSDPConnection *conn = gst_sdp_media_get_connection (media, i);
-
-    if (conn->nettype && conn->addrtype && conn->address) {
-      guint family;
-
-      g_string_append_printf (lines, "c=%s %s %s", conn->nettype,
-          conn->addrtype, conn->address);
-      if (is_multicast_address (conn->address, &family)) {
-        if (family == AF_INET)
-          g_string_append_printf (lines, "/%u", conn->ttl);
-        if (conn->addr_number > 1)
-          g_string_append_printf (lines, "/%u", conn->addr_number);
-      }
-      g_string_append_printf (lines, "\r\n");
-    }
-  }
-
-  for (i = 0; i < gst_sdp_media_bandwidths_len (media); i++) {
-    const GstSDPBandwidth *bandwidth = gst_sdp_media_get_bandwidth (media, i);
-
-    g_string_append_printf (lines, "b=%s:%u\r\n", bandwidth->bwtype,
-        bandwidth->bandwidth);
-  }
-
-  if (media->key.type) {
-    g_string_append_printf (lines, "k=%s", media->key.type);
-    if (media->key.data)
-      g_string_append_printf (lines, ":%s", media->key.data);
-    g_string_append_printf (lines, "\r\n");
-  }
-
-  for (i = 0; i < gst_sdp_media_attributes_len (media); i++) {
-    const GstSDPAttribute *attr = gst_sdp_media_get_attribute (media, i);
-
-    if (attr->key) {
-      g_string_append_printf (lines, "a=%s", attr->key);
-      if (attr->value)
-        g_string_append_printf (lines, ":%s", attr->value);
-      g_string_append_printf (lines, "\r\n");
-    }
-  }
-
-  return g_string_free (lines, FALSE);
-}
-
 /**
  * gst_sdp_media_get_media:
  * @media: a #GstSDPMedia
@@ -1246,7 +936,7 @@ gst_sdp_media_set_media (GstSDPMedia * media, const gchar * med)
  *
  * Returns: the port number of @media.
  */
-guint
+gint
 gst_sdp_media_get_port (const GstSDPMedia * media)
 {
   return media->port;
@@ -1260,7 +950,7 @@ gst_sdp_media_get_port (const GstSDPMedia * media)
  *
  * Returns: the number of ports for @media.
  */
-guint
+gint
 gst_sdp_media_get_num_ports (const GstSDPMedia * media)
 {
   return media->num_ports;
@@ -1277,7 +967,7 @@ gst_sdp_media_get_num_ports (const GstSDPMedia * media)
  * Returns: #GST_SDP_OK.
  */
 GstSDPResult
-gst_sdp_media_set_port_info (GstSDPMedia * media, guint port, guint num_ports)
+gst_sdp_media_set_port_info (GstSDPMedia * media, gint port, gint num_ports)
 {
   media->port = port;
   media->num_ports = num_ports;
@@ -1325,7 +1015,7 @@ gst_sdp_media_set_proto (GstSDPMedia * media, const gchar * proto)
  *
  * Returns: the number of formats in @media.
  */
-guint
+gint
 gst_sdp_media_formats_len (const GstSDPMedia * media)
 {
   return media->fmts->len;
@@ -1409,10 +1099,10 @@ gst_sdp_media_set_information (GstSDPMedia * media, const gchar * information)
  *
  * Returns: the number of connections in @media.
  */
-guint
+gint
 gst_sdp_media_connections_len (const GstSDPMedia * media)
 {
-  return media->connections->len;
+  return (media->connections->len);
 }
 
 /**
@@ -1446,7 +1136,7 @@ gst_sdp_media_get_connection (const GstSDPMedia * media, guint idx)
  */
 GstSDPResult
 gst_sdp_media_add_connection (GstSDPMedia * media, const gchar * nettype,
-    const gchar * addrtype, const gchar * address, guint ttl, guint addr_number)
+    const gchar * addrtype, const gchar * address, gint ttl, gint addr_number)
 {
   GstSDPConnection conn;
 
@@ -1469,10 +1159,10 @@ gst_sdp_media_add_connection (GstSDPMedia * media, const gchar * nettype,
  *
  * Returns: the number of bandwidths in @media.
  */
-guint
+gint
 gst_sdp_media_bandwidths_len (const GstSDPMedia * media)
 {
-  return media->bandwidths->len;
+  return (media->bandwidths->len);
 }
 
 /**
@@ -1485,7 +1175,7 @@ gst_sdp_media_bandwidths_len (const GstSDPMedia * media)
  * Returns: the #GstSDPBandwidth at position @idx.
  */
 const GstSDPBandwidth *
-gst_sdp_media_get_bandwidth (const GstSDPMedia * media, guint idx)
+gst_sdp_media_get_badwidth (const GstSDPMedia * media, guint idx)
 {
   return &g_array_index (media->bandwidths, GstSDPBandwidth, idx);
 }
@@ -1502,7 +1192,7 @@ gst_sdp_media_get_bandwidth (const GstSDPMedia * media, guint idx)
  */
 GstSDPResult
 gst_sdp_media_add_bandwidth (GstSDPMedia * media, const gchar * bwtype,
-    guint bandwidth)
+    gint bandwidth)
 {
   GstSDPBandwidth bw;
 
@@ -1558,10 +1248,10 @@ gst_sdp_media_get_key (const GstSDPMedia * media)
  *
  * Returns: the number of attributes in @media.
  */
-guint
+gint
 gst_sdp_media_attributes_len (const GstSDPMedia * media)
 {
-  return media->attributes->len;
+  return (media->attributes->len);
 }
 
 /**
@@ -1694,7 +1384,7 @@ enum
 
 typedef struct
 {
-  guint state;
+  gint state;
   GstSDPMessage *msg;
   GstSDPMedia *media;
 } SDPContext;
@@ -1705,8 +1395,8 @@ gst_sdp_parse_line (SDPContext * c, gchar type, gchar * buffer)
   gchar str[8192];
   gchar *p = buffer;
 
-#define READ_STRING(field) read_string (str, sizeof (str), &p); REPLACE_STRING (field, str)
-#define READ_UINT(field) read_string (str, sizeof (str), &p); field = strtoul (str, NULL, 10)
+#define READ_STRING(field) read_string (str, sizeof(str), &p);REPLACE_STRING (field, str);
+#define READ_INT(field) read_string (str, sizeof(str), &p);field = atoi(str);
 
   switch (type) {
     case 'v':
@@ -1742,38 +1432,17 @@ gst_sdp_parse_line (SDPContext * c, gchar type, gchar * buffer)
       gst_sdp_message_add_phone (c->msg, buffer);
       break;
     case 'c':
-    {
-      GstSDPConnection conn;
-      gchar *str2;
-
-      memset (&conn, 0, sizeof (conn));
-
-      str2 = p;
-      while ((str2 = strchr (str2, '/')))
-        *str2++ = ' ';
-      READ_STRING (conn.nettype);
-      READ_STRING (conn.addrtype);
-      READ_STRING (conn.address);
-      READ_UINT (conn.ttl);
-      READ_UINT (conn.addr_number);
-
-      if (c->state == SDP_SESSION) {
-        gst_sdp_message_set_connection (c->msg, conn.nettype, conn.addrtype,
-            conn.address, conn.ttl, conn.addr_number);
-      } else {
-        gst_sdp_media_add_connection (c->media, conn.nettype, conn.addrtype,
-            conn.address, conn.ttl, conn.addr_number);
-      }
-      gst_sdp_connection_init (&conn);
+      READ_STRING (c->msg->connection.nettype);
+      READ_STRING (c->msg->connection.addrtype);
+      READ_STRING (c->msg->connection.address);
+      READ_INT (c->msg->connection.ttl);
+      READ_INT (c->msg->connection.addr_number);
       break;
-    }
     case 'b':
     {
       gchar str2[MAX_LINE_LEN];
 
       read_string_del (str, sizeof (str), ':', &p);
-      if (*p != '\0')
-        p++;
       read_string (str2, sizeof (str2), &p);
       if (c->state == SDP_SESSION)
         gst_sdp_message_add_bandwidth (c->msg, str, atoi (str2));
@@ -1784,6 +1453,7 @@ gst_sdp_parse_line (SDPContext * c, gchar type, gchar * buffer)
     case 't':
       break;
     case 'k':
+
       break;
     case 'a':
       read_string_del (str, sizeof (str), ':', &p);
@@ -1803,7 +1473,6 @@ gst_sdp_parse_line (SDPContext * c, gchar type, gchar * buffer)
       memset (&nmedia, 0, sizeof (nmedia));
       gst_sdp_media_init (&nmedia);
 
-      /* m=<media> <port>/<number of ports> <proto> <fmt> ... */
       READ_STRING (nmedia.media);
       read_string (str, sizeof (str), &p);
       slash = g_strrstr (str, "/");
@@ -1896,10 +1565,10 @@ gst_sdp_message_parse_buffer (const guint8 * data, guint size,
 static void
 print_media (GstSDPMedia * media)
 {
-  g_print ("   media:       '%s'\n", GST_STR_NULL (media->media));
-  g_print ("   port:        '%u'\n", media->port);
-  g_print ("   num_ports:   '%u'\n", media->num_ports);
-  g_print ("   proto:       '%s'\n", GST_STR_NULL (media->proto));
+  g_print ("   media:       '%s'\n", media->media);
+  g_print ("   port:        '%d'\n", media->port);
+  g_print ("   num_ports:   '%d'\n", media->num_ports);
+  g_print ("   proto:       '%s'\n", media->proto);
   if (media->fmts->len > 0) {
     guint i;
 
@@ -1908,37 +1577,10 @@ print_media (GstSDPMedia * media)
       g_print ("    format  '%s'\n", g_array_index (media->fmts, gchar *, i));
     }
   }
-  g_print ("   information: '%s'\n", GST_STR_NULL (media->information));
-  if (media->connections->len > 0) {
-    guint i;
-
-    g_print ("   connections:\n");
-    for (i = 0; i < media->connections->len; i++) {
-      GstSDPConnection *conn =
-          &g_array_index (media->connections, GstSDPConnection, i);
-
-      g_print ("    nettype:      '%s'\n", GST_STR_NULL (conn->nettype));
-      g_print ("    addrtype:     '%s'\n", GST_STR_NULL (conn->addrtype));
-      g_print ("    address:      '%s'\n", GST_STR_NULL (conn->address));
-      g_print ("    ttl:          '%u'\n", conn->ttl);
-      g_print ("    addr_number:  '%u'\n", conn->addr_number);
-    }
-  }
-  if (media->bandwidths->len > 0) {
-    guint i;
-
-    g_print ("   bandwidths:\n");
-    for (i = 0; i < media->bandwidths->len; i++) {
-      GstSDPBandwidth *bw =
-          &g_array_index (media->bandwidths, GstSDPBandwidth, i);
-
-      g_print ("    type:         '%s'\n", GST_STR_NULL (bw->bwtype));
-      g_print ("    bandwidth:    '%u'\n", bw->bandwidth);
-    }
-  }
+  g_print ("   information: '%s'\n", media->information);
   g_print ("   key:\n");
-  g_print ("    type:       '%s'\n", GST_STR_NULL (media->key.type));
-  g_print ("    data:       '%s'\n", GST_STR_NULL (media->key.data));
+  g_print ("    type:       '%s'\n", media->key.type);
+  g_print ("    data:       '%s'\n", media->key.data);
   if (media->attributes->len > 0) {
     guint i;
 
@@ -1961,22 +1603,22 @@ print_media (GstSDPMedia * media)
  * Returns: a #GstSDPResult.
  */
 GstSDPResult
-gst_sdp_message_dump (const GstSDPMessage * msg)
+gst_sdp_message_dump (GstSDPMessage * msg)
 {
   g_return_val_if_fail (msg != NULL, GST_SDP_EINVAL);
 
   g_print ("sdp packet %p:\n", msg);
-  g_print (" version:       '%s'\n", GST_STR_NULL (msg->version));
+  g_print (" version:       '%s'\n", msg->version);
   g_print (" origin:\n");
-  g_print ("  username:     '%s'\n", GST_STR_NULL (msg->origin.username));
-  g_print ("  sess_id:      '%s'\n", GST_STR_NULL (msg->origin.sess_id));
-  g_print ("  sess_version: '%s'\n", GST_STR_NULL (msg->origin.sess_version));
-  g_print ("  nettype:      '%s'\n", GST_STR_NULL (msg->origin.nettype));
-  g_print ("  addrtype:     '%s'\n", GST_STR_NULL (msg->origin.addrtype));
-  g_print ("  addr:         '%s'\n", GST_STR_NULL (msg->origin.addr));
-  g_print (" session_name:  '%s'\n", GST_STR_NULL (msg->session_name));
-  g_print (" information:   '%s'\n", GST_STR_NULL (msg->information));
-  g_print (" uri:           '%s'\n", GST_STR_NULL (msg->uri));
+  g_print ("  username:     '%s'\n", msg->origin.username);
+  g_print ("  sess_id:      '%s'\n", msg->origin.sess_id);
+  g_print ("  sess_version: '%s'\n", msg->origin.sess_version);
+  g_print ("  nettype:      '%s'\n", msg->origin.nettype);
+  g_print ("  addrtype:     '%s'\n", msg->origin.addrtype);
+  g_print ("  addr:         '%s'\n", msg->origin.addr);
+  g_print (" session_name:  '%s'\n", msg->session_name);
+  g_print (" information:   '%s'\n", msg->information);
+  g_print (" uri:           '%s'\n", msg->uri);
 
   if (msg->emails->len > 0) {
     guint i;
@@ -1995,26 +1637,14 @@ gst_sdp_message_dump (const GstSDPMessage * msg)
     }
   }
   g_print (" connection:\n");
-  g_print ("  nettype:      '%s'\n", GST_STR_NULL (msg->connection.nettype));
-  g_print ("  addrtype:     '%s'\n", GST_STR_NULL (msg->connection.addrtype));
-  g_print ("  address:      '%s'\n", GST_STR_NULL (msg->connection.address));
-  g_print ("  ttl:          '%u'\n", msg->connection.ttl);
-  g_print ("  addr_number:  '%u'\n", msg->connection.addr_number);
-  if (msg->bandwidths->len > 0) {
-    guint i;
-
-    g_print (" bandwidths:\n");
-    for (i = 0; i < msg->bandwidths->len; i++) {
-      GstSDPBandwidth *bw =
-          &g_array_index (msg->bandwidths, GstSDPBandwidth, i);
-
-      g_print ("  type:         '%s'\n", GST_STR_NULL (bw->bwtype));
-      g_print ("  bandwidth:    '%u'\n", bw->bandwidth);
-    }
-  }
+  g_print ("  nettype:      '%s'\n", msg->connection.nettype);
+  g_print ("  addrtype:     '%s'\n", msg->connection.addrtype);
+  g_print ("  address:      '%s'\n", msg->connection.address);
+  g_print ("  ttl:          '%d'\n", msg->connection.ttl);
+  g_print ("  addr_number:  '%d'\n", msg->connection.addr_number);
   g_print (" key:\n");
-  g_print ("  type:         '%s'\n", GST_STR_NULL (msg->key.type));
-  g_print ("  data:         '%s'\n", GST_STR_NULL (msg->key.data));
+  g_print ("  type:         '%s'\n", msg->key.type);
+  g_print ("  data:         '%s'\n", msg->key.data);
   if (msg->attributes->len > 0) {
     guint i;
 
@@ -2031,7 +1661,7 @@ gst_sdp_message_dump (const GstSDPMessage * msg)
 
     g_print (" medias:\n");
     for (i = 0; i < msg->medias->len; i++) {
-      g_print ("  media %u:\n", i);
+      g_print ("  media %d:\n", i);
       print_media (&g_array_index (msg->medias, GstSDPMedia, i));
     }
   }
