@@ -51,7 +51,8 @@ typedef enum
   GST_SUB_PARSE_FORMAT_MPSUB = 3,
   GST_SUB_PARSE_FORMAT_SAMI = 4,
   GST_SUB_PARSE_FORMAT_TMPLAYER = 5,
-  GST_SUB_PARSE_FORMAT_MPL2 = 6
+  GST_SUB_PARSE_FORMAT_MPL2 = 6,
+  GST_SUB_PARSE_FORMAT_SUBVIEWER = 7
 } GstSubParseFormat;
 
 typedef struct {
