@@ -44,13 +44,13 @@
 #define TCP_BACKLOG             5
 
 /* elementfactory information */
-static GstElementDetails gst_tcp_server_sink_details =
-GST_ELEMENT_DETAILS ("TCP Server sink",
+static const GstElementDetails gst_tcp_server_sink_details =
+GST_ELEMENT_DETAILS ("TCP server sink",
     "Sink/Network",
     "Send data as a server over the network via TCP",
     "Thomas Vander Stichele <thomas at apestaart dot org>");
 
-GST_DEBUG_CATEGORY (tcpserversink_debug);
+GST_DEBUG_CATEGORY_STATIC (tcpserversink_debug);
 #define GST_CAT_DEFAULT (tcpserversink_debug)
 
 enum
@@ -99,10 +99,10 @@ gst_tcp_server_sink_class_init (GstTCPServerSinkClass * klass)
   gobject_class->get_property = gst_tcp_server_sink_get_property;
   gobject_class->finalize = gst_tcp_server_sink_finalize;
 
-  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_HOST,
+  g_object_class_install_property (gobject_class, ARG_HOST,
       g_param_spec_string ("host", "host", "The host/IP to send the packets to",
           TCP_DEFAULT_HOST, G_PARAM_READWRITE));
-  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_PORT,
+  g_object_class_install_property (gobject_class, ARG_PORT,
       g_param_spec_int ("port", "port", "The port to send the packets to",
           0, TCP_HIGHEST_PORT, TCP_DEFAULT_PORT, G_PARAM_READWRITE));
 
@@ -132,6 +132,8 @@ gst_tcp_server_sink_finalize (GObject * gobject)
   GstTCPServerSink *this = GST_TCP_SERVER_SINK (gobject);
 
   g_free (this->host);
+
+  G_OBJECT_CLASS (parent_class)->finalize (gobject);
 }
 
 /* handle a read request on the server,
